/* global Readability, chrome */


/* This is the user file part of reader mode, it's injected after the
   browserjs part of reader mode and does the real injecetion
   (browserjs only does detection at an earlier stage so that we don't
   need to wait for the page to load to update the indicator status of
   the readability of the page */

const FEATURE_HIDE_PAGE = true; /* Hide page on load for better transition while navigating */

function canSetStyle() {
  return document && document.documentElement && document.documentElement.style;
}

// TODO: not working after delaying injection in the browserjs part of reader view
if (FEATURE_HIDE_PAGE && canSetStyle()) {
  document.documentElement.style.display = 'none';
}

const DEBUG_READABILITY = false;

const readabilityCss = `
body {
  color: #111111;
  background-color: #f6f6f6;
  font-family: sans-serif;
  font-size: 1.3em;
  font-style: normal;
  font-weight: 400;
  line-height: 1.5;
  margin: 0 auto;
  max-width: 700px;
  padding: 40px 10px 80px;
}

.entry-content * {
  max-width: 100%;
  white-space: normal;
  font-size: inherit;
  line-height: inherit;
  background: transparent;
}
.entry-content p {
  margin: 1.5em 0;
}

h1 {
  font-size: 1.8em;
  line-height: 1.35;
}

h1, h2, h3, h4, h5, h6, b {
  font-weight: 700;
}

hr {
  border-style: solid;
  border-width: 1px 0 0 0;
}

.entry-content img {
  max-width: 100%;
  height: auto;
}

.entry-content code, .entry-content pre {
  font-weight: 300;
  padding: 0.3em 0.4em;
  text-align: left;
  white-space: pre-wrap;
}

iframe, textarea, input {
  display: none;
}
`;

function __vivaldiGetHTML(article) {
  const content = window.DOMPurify.sanitize(article.content);
  return `<!DOCTYPE html><html><head>
<title>${article.title}</title>
<style>${readabilityCss}</style>
</head><body>
<h1>${article.title}</h1>
<hr />
<article class="entry-content">${content}</article>
</body></html>`;
}

function ready() {
  if (
    typeof document !== 'undefined' &&
    ['complete', 'loaded', 'interactive'].indexOf(document.readyState) > -1
  ) {
    return true;
  }
  return false;
}

let _numTriesLeft = 5;
function onLoad() {
  if (!window['Readability'] && _numTriesLeft-- > -1) {
    setTimeout(onLoad, 500);
  }
  try {
    const loc = document.location;

    const uri = {
      spec: loc.href,
      host: loc.host,
      prePath: loc.protocol + '//' + loc.host,
      scheme: loc.protocol.substr(0, loc.protocol.indexOf(':')),
      pathBase: loc.protocol +
        '//' +
        loc.host +
        loc.pathname.substr(0, loc.pathname.lastIndexOf('/') + 1),
    };

    // TODO: should probably reuse this object when enabling, instead
    // of running it again via the user script
    const readability = new Readability(
      uri,
      document.cloneNode(true),
      DEBUG_READABILITY && {debug: true}
    );

    const isProbablyReaderable = readability.isProbablyReaderable();

    let article;

    if (isProbablyReaderable) {
      article = readability.parse();
    } else {
      console.log('[readability] Probably not readable');
    }

    if (DEBUG_READABILITY) {
      console.log('isReadable?', isProbablyReaderable, 'article:', article);
    }

    if (article) {
      // VB-22288 Cancel unload events
      document.beforeunload = () => {};
      document.unload = () => {};

      // TODO: VB-30078 delay empty DOM to reset stuck scrollbar
      // This fix works only for text/html, might need Chrome fix
      if (document.contentType === 'text/html') {
        document.open();
        setTimeout(() => {
          document.write(__vivaldiGetHTML(article));
          document.close();
          window.stop();
          if (chrome && chrome.runtime && chrome.runtime.sendMessage) {
            chrome.runtime.sendMessage({type: 'SET_RM_STYLE'});
          }
        }, 16);
      } else {
        const reader = document.implementation.createHTMLDocument(document.title);
        reader.documentElement.innerHTML = __vivaldiGetHTML(article);
        document.replaceChild(reader.documentElement, document.documentElement);
        if (chrome && chrome.runtime && chrome.runtime.sendMessage) {
          chrome.runtime.sendMessage({type: 'SET_RM_STYLE'});
        }
      }
    } else {
      console.log('[readability] Nothing to show');
    }
  } finally {
    // The page is rendered with display: none, so need to ensure this
    // is always called
    if (FEATURE_HIDE_PAGE && canSetStyle()) {
      document.documentElement.style.display = 'block';
    }
  }
}

if (ready()) {
  onLoad();
} else {
  window.addEventListener('DOMContentLoaded', onLoad, false);
}
